@@ -25,5 +25,9 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
+import compas
+
+if compas.is_grasshopper():
+    from .artists import *  # noqa F403
 
 __all__ = [name for name in dir() if not name.startswith('_')]
