@@ -1,5 +1,9 @@
+from time import time
+
 import autograd.numpy as np
 
+from autograd import grad as agrad
+
 from functools import partial
 
 from compas_cem.data import Data
@@ -8,6 +12,7 @@ from compas_cem.equilibrium import static_equilibrium
 from compas_cem.equilibrium.force_numpy import equilibrium_state_numpy
 
 from compas_cem.optimization import grad_autograd
+from compas_cem.optimization import grad_finite_differences
 from compas_cem.optimization import objective_function_numpy
 from compas_cem.optimization import nlopt_solver
 from compas_cem.optimization import nlopt_status
@@ -31,7 +36,9 @@ class Optimizer(Data):
 
         self.parameters = {}
         self.constraints = {}
+
         self.x_opt = None
+        self.time_opt = None
         self.penalty = None
         self.evals = None
         self.gradient_norm = None
@@ -115,51 +122,69 @@ class Optimizer(Data):
         """
         The objective function to minimize.
         """
-        obj_func = objective_function_numpy
-        func = partial(self._optimize_form, topology=topology, tmax=tmax, eta=eta)
-        return partial(obj_func, x_func=func, grad_func=grad_func)
+        f = objective_function_numpy
+        x_func = partial(self._optimize_form, topology=topology, tmax=tmax, eta=eta)
+        return partial(f, x_func=x_func, grad_func=grad_func)
 
 # ------------------------------------------------------------------------------
 # Gradient Function
 # ------------------------------------------------------------------------------
 
-    def gradient_func(self, topology, tmax, eta):
+    def gradient_func(self, grad_f, topology, tmax, eta, step_size):
         """
         The objective function to calculate gradients from.
         """
         x_func = partial(self._optimize_form, topology=topology, tmax=tmax, eta=eta)
-        return partial(grad_autograd, grad_func=x_func)
+        return partial(grad_f, x_func=x_func, step_size=step_size)
 
 # ------------------------------------------------------------------------------
 # Solver
 # ------------------------------------------------------------------------------
 
-    def solve_nlopt(self, topology, algorithm, iters, eps=None, tmax=100, eta=1e-6):
+    def solve(self, topology, algorithm="SLSQP", grad="AD", step_size=1e-6, iters=100, eps=1e-6, kappa=1e-8, tmax=100, eta=1e-6, verbose=False):
         """
-        Solve an optimization problem with NLOpt.
+        Solve a constrained form-finding problem using gradient-based optimization.
+
+        The gradient of the objective function computed using automatic differentiation.
 
         Parameters
         ----------
         topology : :class:`compas_cem.diagrams.TopologyDiagram`
             A topology diagram.
-        algorithm : ``str``
+        algorithm : ``str``, optional
             The name of the gradient-based local optimization algorithm to use.
             Only the following local gradient-based optimization algorithms are supported:
 
             - SLSQP: Sequential Least Squares Programming
             - LBFGS: Low-Storage Broyden-Fletcher-Goldfarb-Shanno
-            - AUGLAG: Augmented Lagrangian
             - MMA: Method of Moving Asymptotes
             - TNEWTON: Preconditioned Truncated Newton
+            - AUGLAG: Augmented Lagrangian
+            - VAR: Limited-Memory Variable-Metric Algorithm
 
+            Defaults to "SLSQP".
             Refer to the NLopt `documentation <https://nlopt.readthedocs.io/en/latest/>`_ for more details on their theoretical underpinnings.
-        iters : ``int``
+        grad : ``str``, optional
+            The method to compute the gradient of the objective function.
+            The currently available methods are:
+
+            - AD: Automatic differentiation.
+            - FD: Finite differences.
+
+            Defaults to "AD".
+        iters : ``int``, optional
             The maximum number of iterations to run the optimization algorithm for.
+            Defaults to ``100``.
         eps : ``float``, optional
-            The numerical convergence threshold for the optimization algorithm.
-            If value is set to ``None``, this parameter is ignored and the
-            optimization algorithm will run until ``iters`` is exhausted.
-            Defaults to ``None``.
+            The convergence threshold for the output value of the objective function.
+            Defaults to ``1e-6``.
+        kappa : ``float``, optional
+            The convergence threshold for the norm of the gradient of the objective function.
+            Defaults to ``1e-8``.
+        step_size : ``float``, optional
+            The step size to calculate the gradient of the objective function via finite differences.
+            It becomes active only if ``grad="FD"``. It is otherwise ignored by this function.
+            Defaults to ``1e-3``.
         tmax : ``int``, optional
             The maximum number of iterations the CEM form-finding algorithm will run for.
             If ``eta`` is hit first, the form-finding algorithm will stop early.
@@ -168,19 +193,39 @@ class Optimizer(Data):
             The numerical converge threshold of the CEM form-finding algorithm.
             If ``tmax`` is hit first, the form-finding algorithm will stop early.
             Defaults to ``1e-6``.
+        verbose : ``bool``, optional
+            A flag to prints statistics of the optimization process.
+            Defaults to ``True``.
 
         Returns
         -------
         form : :class:`compas_cem.diagrams.FormDiagram`
             A form diagram.
         """
+        if verbose:
+            print("----------")
+            print("Optimization with {} started!".format(algorithm))
+            print(f"# Parameters: {self.number_of_parameters()}, # Constraints {self.number_of_constraints()}")
+
         # test for bad stuff before going any further
         self.check_optimization_sanity()
 
         # compose gradient and objective functions
-        topology_b = topology.copy()
-        grad_func = self.gradient_func(topology_b, tmax, eta)
-        penalty_func = self.objective_func(topology, grad_func, tmax, eta)
+        if grad not in ("AD", "FD"):
+            raise ValueError(f"Gradient method {grad} is not supported!")
+        if grad == "AD":
+            if verbose:
+                print("Computing gradients using automatic differentiation!")
+            x_func = partial(self._optimize_form, topology=topology.copy(), tmax=tmax, eta=eta)
+            grad_func = partial(grad_autograd, grad_func=agrad(x_func))  # x, grad, x_func
+
+        elif grad == "FD":
+            if verbose:
+                print(f"Warning: Calculating gradients using finite differences with step size {step_size}. This may take a while...")
+            grad_func = self.gradient_func(grad_finite_differences, topology.copy(), tmax, eta, step_size)
+
+        # grad_func = self.gradient_func(grad_func, topology.copy(), tmax, eta, step_size)
+        obj_func = self.objective_func(topology, grad_func, tmax, eta)
 
         # generate optimization variables
         x = self.optimization_parameters(topology)
@@ -189,47 +234,59 @@ class Optimizer(Data):
         bounds_low, bounds_up = self.optimization_bounds(topology)
 
         # stack keyword arguments
-        hyper_parameters = {"f": penalty_func,
+        hyper_parameters = {"f": obj_func,
                             "algorithm": algorithm,
                             "dims": self.number_of_parameters(),
                             "bounds_low": bounds_low,
                             "bounds_up": bounds_up,
                             "iters": iters,
                             "eps": eps,
-                            "ftol": None}
+                            "ftol": kappa}
 
         # assemble optimization solver
         solver = nlopt_solver(**hyper_parameters)
 
         # solve optimization problem
         x_opt = None
+        start = time()
         try:
             x_opt = solver.optimize(x)
-            evals = solver.get_numevals()
-            print("Optimization ended correctly!")
-            print("Number of evaluations incurred: {}".format(evals))
+            if verbose:
+                print("Optimization ended correctly!")
         except RoundoffLimited:
             print("Optimization was halted because roundoff errors limited progress")
-            print("Returned results are generally still useful though!")
+            print("Results may still be useful though!")
             x_opt = self.optimization_parameters(topology)
+        except RuntimeError:
+            print("Optimization failed for reasons I do not understand yet...")
+            print(f"Optimization total runtime: {round(time() - start, 4)} seconds")
+            return static_equilibrium(topology)
 
         # fetch last optimum value of loss function
+        time_opt = time() - start
         loss_opt = solver.last_optimum_value()
         evals = solver.get_numevals()
         status = nlopt_status(solver.last_optimize_result())
 
         # set optimizer attributes
+        self.time_opt = time_opt
         self.x_opt = x_opt
         self.penalty = loss_opt
         self.evals = evals
         self.status = status
 
         # set norm of the gradient
-        # np.zeros is a dummy array (signature requirement set by nlopt)
+        # NOTE: np.zeros is a dummy array (signature requirement set by nlopt)
         self.gradient = grad_func(x_opt, np.zeros(x_opt.size))
         self.gradient_norm = np.linalg.norm(self.gradient)
 
-        print("Optimization status: {}".format(status))
+        if verbose:
+            print(f"Optimization total runtime: {round(time_opt, 6)} seconds")
+            print("Number of evaluations incurred: {}".format(evals))
+            print(f"Final value of the objective function: {round(loss_opt, 6)}")
+            print(f"Norm of the gradient of the objective function: {round(self.gradient_norm, 6)}")
+            print(f"Optimization status: {status}".format(status))
+            print("----------")
 
         # exit like a champion
         return static_equilibrium(topology)
