@@ -1,5 +1,3 @@
-from math import fabs
-
 from compas_cem.optimization.parameters import Parameter
 
 
@@ -19,12 +17,6 @@ class NodeParameter(Parameter):
     def __init__(self, key, bound_low, bound_up, **kwargs):
         super(NodeParameter, self).__init__(key, bound_low, bound_up, **kwargs)
 
-    def key(self):
-        """
-        The node key.
-        """
-        return self._key
-
     def start_value(self, topology):
         """
         The initial value of the node optimization parameter.
@@ -32,38 +24,6 @@ class NodeParameter(Parameter):
         val = topology.node_attribute(key=self.key(), name=self._attr_name)
         return val
 
-    def bound_low(self, topology):
-        """
-        The lower bound of the node optimization parameter.
-
-        Notes
-        -----
-        Calculated as the initial parameter minus this bound's absolute value.
-        """
-        return self.start_value(topology) - fabs(self._bound_low)
-
-    def bound_up(self, topology):
-        """
-        The upper bound of the node optimization parameter.
-
-        Notes
-        -----
-        Calculated as the initial parameter plus this bound's absolute value.
-        """
-        return self.start_value(topology) + fabs(self._bound_up)
-
-    def attr_name(self):
-        """
-        The name of the node attribute to parametrize.
-        """
-        return self._attr_name
-
-    def __repr__(self):
-        """
-        """
-        st = "{0}(key={1!r}, bound_low={2!r}, bound_up={3!r})"
-        return st.format(self.__class__.__name__, self._key, self._bound_low, self._bound_up)
-
 # ------------------------------------------------------------------------------
 # Data
 # ------------------------------------------------------------------------------
