@@ -1,14 +1,11 @@
-from time import time
-
-
-__all__ = ["solve_nlopt_proxy"]
+__all__ = ["solve_proxy"]
 
 
 # ------------------------------------------------------------------------------
 # Optimization
 # ------------------------------------------------------------------------------
 
-def solve_nlopt_proxy(topology, constraints, parameters, algorithm, iters, eps=None, tmax=100, eta=1e-6):
+def solve_proxy(topology, constraints, parameters, algorithm, iters, eps=None, tmax=100, eta=1e-6):
     """
     Solve a constrained form-finding task through a Proxy server hyperspace tunnel.
 
@@ -49,8 +46,10 @@ def solve_nlopt_proxy(topology, constraints, parameters, algorithm, iters, eps=N
 
     Returns
     -------
+    topology : :class:`compas_cem.diagrams.TopologyDiagram`
+        The topology diagram with optimal parameters as found by the optimization algorithm.
     form : :class:`compas_cem.diagrams.FormDiagram`
-        A form diagram.
+        The constrained form diagram.
     objective : `float`
         The final value of the objective function.
     grad_norm : `float`
@@ -75,16 +74,20 @@ def solve_nlopt_proxy(topology, constraints, parameters, algorithm, iters, eps=N
     for parameter in parameters:
         optimizer.add_parameter(parameter)
 
-    start = time()
-    form = optimizer.solve_nlopt(topology, algorithm, iters, eps, tmax, eta)
+    form = optimizer.solve(topology=topology,
+                           algorithm=algorithm,
+                           iters=iters,
+                           eps=eps,
+                           tmax=tmax,
+                           eta=eta)
 
-    duration = round(time() - start, 2)
+    duration = optimizer.time_opt
     objective = optimizer.penalty
     evals = optimizer.evals
     grad_norm = optimizer.gradient_norm
     status = optimizer.status
 
-    return form, objective, grad_norm, evals, duration, status
+    return topology, form, objective, grad_norm, evals, duration, status
 
 # ------------------------------------------------------------------------------
 # Main
