@@ -5,7 +5,7 @@ compas_cem.plotters
 .. currentmodule:: compas_cem.plotters
 
 
-Classes
+Artists
 =======
 
 .. autosummary::
@@ -14,6 +14,15 @@ Classes
 
     FormArtist
     TopologyArtist
+
+Plotters
+========
+
+.. autosummary::
+    :toctree: generated/
+    :nosignatures:
+
+    Plotter
 """
 
 from __future__ import absolute_import
@@ -24,6 +33,7 @@ from __future__ import print_function
 # from .<module> import *
 from .formartist import *  # noqa F403
 from .topologyartist import *  # noqa F403
+from .plotter import *  # noqa F403
 # from .proxy import *  # noqa F403
 
 __all__ = [name for name in dir() if not name.startswith('_')]
